<?php

namespace Drupal\gotem\EventSubscriber;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\gotem\Event\ContentModerationEvent;
use Drupal\gotem\Moderation\ModerationInterface;
use Drupal\gotem\Service\ThresholdService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\node\NodeInterface;

/**
 * Class ContentModerationSubscriber. Handles content moderation events.
 */
class ContentModerationSubscriber implements EventSubscriberInterface {

  /**
   * The moderation service.
   *
   * @var \Drupal\gotem\Moderation\ModerationInterface
   */
  protected $moderationService;

  /**
   * The threshold service.
   *
   * @var \Drupal\gotem\Service\ThresholdService
   */
  protected $thresholdService;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Constructs a new ContentModerationSubscriber.
   */
  public function __construct(ModerationInterface $moderation_service, ThresholdService $threshold_service, AccountProxyInterface $current_user, MessengerInterface $messenger, LoggerChannelFactoryInterface $logger) {
    $this->moderationService = $moderation_service;
    $this->thresholdService = $threshold_service;
    $this->currentUser = $current_user;
    $this->messenger = $messenger;
    $this->logger = $logger->get('gotem');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ContentModerationEvent::EVENT_NAME => 'onContentModeration',
    ];
  }

  /**
   * Responds to content moderation event.
   */
  public function onContentModeration(ContentModerationEvent $event) {
    $entity = $event->getEntity();

    if ($entity instanceof NodeInterface) {
      $text_to_check = $this->extractTextFromFields($entity, ['title', 'body']);
      if (empty(trim($text_to_check))) {
        return;
      }

      $result = $this->moderationService->moderateText($text_to_check);

      if ($result) {
        $categories = ['hate', 'violence', 'sexual', 'self-harm'];

        // Check moderation score for each category.
        foreach ($categories as $category) {
          $moderation_score = $result['category_scores'][$category] ?? 0;

          // Flag content if it exceeds the threshold for any category.
          if ($this->thresholdService->shouldFlagContent($moderation_score, $category)) {
            $this->handleFlaggedContent($entity, $result, $category);
            break;
          }
        }
      }
    }
  }

  /**
   * Handles flagged content.
   */
  protected function handleFlaggedContent(NodeInterface $entity, array $result, string $category) {
    $this->messenger->addError($this->t('This content has been flagged for violating the %category threshold.', ['%category' => ucfirst($category)]));
    $this->logger->warning('Content flagged for node ID @id in the @category category.', ['@id' => $entity->id(), '@category' => $category]);
  }

  /**
   * Extracts text from the given fields of the entity.
   */
  protected function extractTextFromFields(NodeInterface $entity, array $fields) {
    $text = '';
    foreach ($fields as $field) {
      $text .= $entity->get($field)->value;
    }
    return $text;
  }

}
